@@ -5,37 +5,33 @@ import importlib
 import importlib.util
 import itertools
 import json
-import operator
 import os
 import site
 import sys
+import typing
+from functools import cached_property
 from pathlib import Path
 from sysconfig import get_paths, get_python_version, get_scheme_names
+from urllib.parse import urlparse
 
 import pipenv
-from pipenv import cmdparse
 from pipenv.patched.pip._internal.commands.install import InstallCommand
 from pipenv.patched.pip._internal.index.package_finder import PackageFinder
-from pipenv.patched.pip._internal.req.req_uninstall import UninstallPathSet
+from pipenv.patched.pip._internal.req.req_install import InstallRequirement
 from pipenv.patched.pip._vendor import pkg_resources
+from pipenv.patched.pip._vendor.packaging.specifiers import SpecifierSet
 from pipenv.patched.pip._vendor.packaging.utils import canonicalize_name
-from pipenv.utils.constants import is_type_checking
+from pipenv.utils import console
+from pipenv.utils.fileutils import normalize_path, temp_path
+from pipenv.utils.funktools import chunked, unnest
 from pipenv.utils.indexes import prepare_pip_source_args
 from pipenv.utils.processes import subprocess_run
-from pipenv.utils.shell import make_posix, normalize_path
-from pipenv.vendor import click, vistir
+from pipenv.utils.shell import make_posix, temp_environ
+from pipenv.vendor.pythonfinder.utils import is_in_path
 
-try:
-    # this is only in Python3.8 and later
-    from functools import cached_property
-except ImportError:
-    # eventually distlib will remove cached property when they drop Python3.7
-    from pipenv.patched.pip._vendor.distlib.util import cached_property
-
-
-if is_type_checking():
+if typing.TYPE_CHECKING:
     from types import ModuleType
-    from typing import ContextManager, Dict, Generator, List, Optional, Set, Union
+    from typing import ContextManager, Generator
 
     from pipenv.project import Project, TPipfile, TSource
     from pipenv.vendor import tomlkit
@@ -46,13 +42,13 @@ BASE_WORKING_SET = pkg_resources.WorkingSet(sys.path)
 class Environment:
     def __init__(
         self,
-        prefix: Optional[str] = None,
-        python: Optional[str] = None,
+        prefix: str | None = None,
+        python: str | None = None,
         is_venv: bool = False,
         base_working_set: pkg_resources.WorkingSet = None,
-        pipfile: Optional[Union[tomlkit.toml_document.TOMLDocument, TPipfile]] = None,
-        sources: Optional[List[TSource]] = None,
-        project: Optional[Project] = None,
+        pipfile: tomlkit.toml_document.TOMLDocument | TPipfile | None = None,
+        sources: list[TSource] | None = None,
+        project: Project | None = None,
     ):
         super().__init__()
         self._modules = {"pkg_resources": pkg_resources, "pipenv": pipenv}
@@ -98,7 +94,7 @@ class Environment:
     @classmethod
     def resolve_dist(
         cls, dist: pkg_resources.Distribution, working_set: pkg_resources.WorkingSet
-    ) -> Set[pkg_resources.Distribution]:
+    ) -> set[pkg_resources.Distribution]:
         """Given a local distribution and a working set, returns all dependencies from the set.
 
         :param dist: A single distribution to find the dependencies of
@@ -143,12 +139,12 @@ class Environment:
             py_version = sysconfig.get_python_version()
             return py_version
 
-    def find_libdir(self) -> Optional[Path]:
+    def find_libdir(self) -> Path | None:
         libdir = self.prefix / "lib"
         return next(iter(list(libdir.iterdir())), None)
 
     @property
-    def python_info(self) -> Dict[str, str]:
+    def python_info(self) -> dict[str, str]:
         include_dir = self.prefix / "include"
         if not os.path.exists(include_dir):
             include_dirs = self.get_include_path()
@@ -187,7 +183,7 @@ class Environment:
             return "posix_prefix"
 
     @cached_property
-    def base_paths(self) -> Dict[str, str]:
+    def base_paths(self) -> dict[str, str]:
         """
         Returns the context appropriate paths for the environment.
 
@@ -197,11 +193,6 @@ class Environment:
         .. note:: The implementation of this is borrowed from a combination of pip and
            virtualenv and is likely to change at some point in the future.
 
-        >>> from pipenv.core import project
-        >>> from pipenv.environment import Environment
-        >>> env = Environment(prefix=project.virtualenv_location, is_venv=True, sources=project.sources)
-        >>> import pprint
-        >>> pprint.pprint(env.base_paths)
         {'PATH': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW/bin::/bin:/usr/bin',
         'PYTHONPATH': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW/lib/python3.7/site-packages',
         'data': '/home/hawk/.virtualenvs/pipenv-MfOPs1lW',
@@ -298,7 +289,7 @@ class Environment:
         return py
 
     @cached_property
-    def sys_path(self) -> List[str]:
+    def sys_path(self) -> list[str]:
         """
         The system path inside the environment
 
@@ -346,16 +337,17 @@ class Environment:
             "value = u'{0}'.format(json.dumps(paths)); print(value)"
         )
         sysconfig_line = "sysconfig.get_path('{0}')"
+
         if python_lib:
-            for key in ("purelib", "platlib", "stdlib", "platstdlib"):
-                pylib_lines.append(
-                    f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
-                )
+            pylib_lines += [
+                f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
+                for key in ("purelib", "platlib", "stdlib", "platstdlib")
+            ]
         if python_inc:
-            for key in ("include", "platinclude"):
-                pyinc_lines.append(
-                    f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
-                )
+            pyinc_lines += [
+                f"u'{key}': u'{{0}}'.format({sysconfig_line.format(key)})"
+                for key in ("include", "platinclude")
+            ]
         lines = pylib_lines + pyinc_lines
         if scripts:
             lines.append(
@@ -369,7 +361,7 @@ class Environment:
         py_command = py_command % lines_as_str
         return py_command
 
-    def get_paths(self) -> Optional[Dict[str, str]]:
+    def get_paths(self) -> dict[str, str] | None:
         """
         Get the paths for the environment by running a subcommand
 
@@ -397,11 +389,11 @@ class Environment:
                     paths[key] = make_posix(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         return None
 
-    def get_lib_paths(self) -> Dict[str, str]:
+    def get_lib_paths(self) -> dict[str, str]:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -420,8 +412,8 @@ class Environment:
                     paths[key] = make_posix(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         if not paths:
             if not self.prefix.joinpath("lib").exists():
                 return {}
@@ -452,7 +444,7 @@ class Environment:
                 return paths
         return {}
 
-    def get_include_path(self) -> Optional[Dict[str, str]]:
+    def get_include_path(self) -> dict[str, str] | None:
         """Get the include path for the environment
 
         :return: The python include path for the environment
@@ -468,8 +460,8 @@ class Environment:
                     paths[key] = make_posix(paths[key])
             return paths
         else:
-            click.secho(f"Failed to load paths: {c.stderr}", fg="yellow")
-            click.secho(f"Output: {c.stdout}", fg="yellow")
+            console.print(f"Failed to load paths: {c.stderr}", style="yellow")
+            console.print(f"Output: {c.stdout}", style="yellow")
         return None
 
     @cached_property
@@ -487,9 +479,9 @@ class Environment:
         return sys_prefix
 
     @cached_property
-    def paths(self) -> Dict[str, str]:
+    def paths(self) -> dict[str, str]:
         paths = {}
-        with vistir.contextmanagers.temp_environ(), vistir.contextmanagers.temp_path():
+        with temp_environ(), temp_path():
             os.environ["PYTHONIOENCODING"] = "utf-8"
             os.environ["PYTHONDONTWRITEBYTECODE"] = "1"
             paths = self.base_paths
@@ -518,34 +510,28 @@ class Environment:
         prefixes = [
             Path(prefix)
             for prefix in self.base_paths["libdirs"].split(os.pathsep)
-            if vistir.path.is_in_path(prefix, self.prefix.as_posix())
+            if is_in_path(prefix, self.prefix.as_posix())
         ]
         for loc in prefixes:
             if not loc.exists():
                 continue
             for pth in loc.iterdir():
-                if not pth.suffix == ".egg-link":
+                if pth.suffix != ".egg-link":
                     continue
                 contents = [
-                    vistir.path.normalize_path(line.strip())
-                    for line in pth.read_text().splitlines()
+                    normalize_path(line.strip()) for line in pth.read_text().splitlines()
                 ]
                 pth.write_text("\n".join(contents))
 
     def get_distributions(self) -> Generator[pkg_resources.Distribution, None, None]:
         """
-        Retrives the distributions installed on the library path of the environment
+        Retrieves the distributions installed on the library path of the environment
 
         :return: A set of distributions found on the library path
         :rtype: iterator
         """
 
-        pip_target_dir = os.environ.get("PIP_TARGET")
-        libdirs = (
-            [pip_target_dir]
-            if pip_target_dir
-            else self.base_paths["libdirs"].split(os.pathsep)
-        )
+        libdirs = self.base_paths["libdirs"].split(os.pathsep)
         dists = (pkg_resources.find_distributions(libdir) for libdir in libdirs)
         yield from itertools.chain.from_iterable(dists)
 
@@ -586,7 +572,7 @@ class Environment:
         location = _normalized(make_posix(location))
         return any(location.startswith(prefix) for prefix in prefixes)
 
-    def get_installed_packages(self) -> List[pkg_resources.Distribution]:
+    def get_installed_packages(self) -> list[pkg_resources.Distribution]:
         """Returns all of the installed packages in a given environment"""
         workingset = self.get_working_set()
         packages = [
@@ -650,7 +636,7 @@ class Environment:
 
     def get_outdated_packages(
         self, pre: bool = False
-    ) -> List[pkg_resources.Distribution]:
+    ) -> list[pkg_resources.Distribution]:
         return [
             pkg
             for pkg in self.get_package_info(pre=pre)
@@ -683,7 +669,7 @@ class Environment:
     def get_package_requirements(self, pkg=None):
         from itertools import chain
 
-        from pipenv.vendor.pipdeptree import PackageDAG
+        from pipenv.vendor.pipdeptree._models import PackageDAG
 
         flatten = chain.from_iterable
 
@@ -693,10 +679,10 @@ class Environment:
 
         tree = PackageDAG.from_pkgs(packages).sort()
         branch_keys = {r.key for r in flatten(tree.values())}
-        if pkg is not None:
-            nodes = [p for p in tree.keys() if p.key == pkg]
+        if pkg is None:
+            nodes = [p for p in tree if p.key not in branch_keys]
         else:
-            nodes = [p for p in tree.keys() if p.key not in branch_keys]
+            nodes = [p for p in tree if p.key == pkg]
         key_tree = {k.key: v for k, v in tree.items()}
 
         return [self._get_requirements_for_package(p, key_tree) for p in nodes]
@@ -716,8 +702,6 @@ class Environment:
         yield new_node
 
     def reverse_dependencies(self):
-        from vistir.misc import chunked, unnest
-
         rdeps = {}
         for req in self.get_package_requirements():
             for d in self.reverse_dependency(req):
@@ -765,85 +749,39 @@ class Environment:
 
         return any(d for d in self.get_distributions() if d.project_name == pkgname)
 
-    def is_satisfied(self, req):
+    def is_satisfied(self, req: InstallRequirement):
         match = next(
             iter(
                 d
                 for d in self.get_distributions()
-                if canonicalize_name(d.project_name) == req.normalized_name
+                if req.name
+                and canonicalize_name(d.project_name) == canonicalize_name(req.name)
             ),
             None,
         )
         if match is not None:
-            if req.editable and req.line_instance.is_local and self.find_egg(match):
-                requested_path = req.line_instance.path
-                return requested_path and os.path.samefile(requested_path, match.location)
-            elif match.has_metadata("direct_url.json"):
-                direct_url_metadata = json.loads(match.get_metadata("direct_url.json"))
-                commit_id = direct_url_metadata.get("vcs_info", {}).get("commit_id", "")
-                vcs_type = direct_url_metadata.get("vcs_info", {}).get("vcs", "")
-                _, pipfile_part = req.as_pipfile().popitem()
-                return (
-                    vcs_type == req.vcs
-                    and commit_id == req.commit_hash
-                    and direct_url_metadata["url"] == pipfile_part[req.vcs]
-                )
-            elif req.is_vcs or req.is_file_or_url:
-                return False
-            elif req.line_instance.specifiers is not None:
-                return req.line_instance.specifiers.contains(
+            if req.specifier is not None:
+                return SpecifierSet(str(req.specifier)).contains(
                     match.version, prereleases=True
                 )
+            if req.link is None:
+                return True
+            elif req.editable and req.link.is_file:
+                requested_path = req.link.file_path
+                if os.path.exists(requested_path):
+                    local_path = requested_path
+                else:
+                    parsed_url = urlparse(requested_path)
+                    local_path = parsed_url.path
+                return requested_path and os.path.samefile(local_path, match.location)
+            elif match.has_metadata("direct_url.json") or (req.link and req.link.is_vcs):
+                # Direct URL installs and VCS installs we assume are not satisfied
+                # since due to skip-lock we may be installing from Pipfile we have insufficient
+                # information to determine if a branch or ref has actually changed.
+                return False
             return True
         return False
 
-    def run(self, cmd, cwd=os.curdir):
-        """Run a command with :class:`~subprocess.Popen` in the context of the environment
-
-        :param cmd: A command to run in the environment
-        :type cmd: str or list
-        :param str cwd: The working directory in which to execute the command, defaults to :data:`os.curdir`
-        :return: A finished command object
-        :rtype: :class:`~subprocess.Popen`
-        """
-
-        c = None
-        with self.activated():
-            script = cmdparse.Script.parse(cmd)
-            c = vistir.misc.run(
-                script._parts,
-                return_object=True,
-                nospin=True,
-                cwd=cwd,
-                write_to_stdout=False,
-            )
-        return c
-
-    def run_py(self, cmd, cwd=os.curdir):
-        """Run a python command in the environment context.
-
-        :param cmd: A command to run in the environment - runs with `python -c`
-        :type cmd: str or list
-        :param str cwd: The working directory in which to execute the command, defaults to :data:`os.curdir`
-        :return: A finished command object
-        :rtype: :class:`~subprocess.Popen`
-        """
-
-        c = None
-        if isinstance(cmd, str):
-            script = cmdparse.Script.parse(f"{self.python} -c {cmd}")
-        else:
-            script = cmdparse.Script.parse([self.python, "-c"] + list(cmd))
-        with self.activated():
-            c = vistir.misc.run(
-                script._parts,
-                return_object=True,
-                nospin=True,
-                cwd=cwd,
-                write_to_stdout=False,
-            )
-        return c
-
     def run_activate_this(self):
         """Runs the environment's inline activation script"""
         if self.is_venv:
@@ -852,7 +790,7 @@ class Environment:
                 raise OSError(f"No such file: {activate_this!s}")
             with open(activate_this) as f:
                 code = compile(f.read(), activate_this, "exec")
-                exec(code, dict(__file__=activate_this))
+                exec(code, {"__file__": activate_this})
 
     @contextlib.contextmanager
     def activated(self):
@@ -877,7 +815,7 @@ class Environment:
         original_path = sys.path
         original_prefix = sys.prefix
         prefix = self.prefix.as_posix()
-        with vistir.contextmanagers.temp_environ(), vistir.contextmanagers.temp_path():
+        with temp_environ(), temp_path():
             os.environ["PATH"] = os.pathsep.join(
                 [
                     self.script_basedir,
@@ -903,103 +841,3 @@ class Environment:
             finally:
                 sys.path = original_path
                 sys.prefix = original_prefix
-
-    @cached_property
-    def finders(self):
-        from pipenv.vendor.pythonfinder import Finder
-
-        finders = [
-            Finder(path=self.base_paths["scripts"], global_search=gs, system=False)
-            for gs in (False, True)
-        ]
-        return finders
-
-    @property
-    def finder(self):
-        return next(iter(self.finders), None)
-
-    def which(self, search, as_path=True):
-        find = operator.methodcaller("which", search)
-        result = next(iter(filter(None, (find(finder) for finder in self.finders))), None)
-        if not result:
-            result = self._which(search)
-        else:
-            if as_path:
-                result = str(result.path)
-        return result
-
-    def install(self, requirements):
-        if not isinstance(requirements, (tuple, list)):
-            requirements = [requirements]
-        with self.get_finder() as finder:
-            args = []
-            for format_control in ("no_binary", "only_binary"):
-                formats = getattr(finder.format_control, format_control)
-                args.extend(
-                    (
-                        "--" + format_control.replace("_", "-"),
-                        ",".join(sorted(formats or {":none:"})),
-                    )
-                )
-            if finder.index_urls:
-                args.extend(["-i", finder.index_urls[0]])
-                for extra_index in finder.index_urls[1:]:
-                    args.extend(["--extra-index-url", extra_index])
-            else:
-                args.append("--no-index")
-            for link in finder.find_links:
-                args.extend(["--find-links", link])
-            for _, host, _ in finder.secure_origins:
-                args.extend(["--trusted-host", host])
-            if finder.allow_all_prereleases:
-                args.append("--pre")
-            if finder.process_dependency_links:
-                args.append("--process-dependency-links")
-            args.append("--")
-            args.extend(requirements)
-            out, _ = vistir.misc.run(
-                args, return_object=False, nospin=True, block=True, combine_stderr=False
-            )
-
-    @contextlib.contextmanager
-    def uninstall(self, pkgname, *args, **kwargs):
-        """A context manager which allows uninstallation of packages from the environment
-
-        :param str pkgname: The name of a package to uninstall
-
-        >>> env = Environment("/path/to/env/root")
-        >>> with env.uninstall("pytz", auto_confirm=True, verbose=False) as uninstaller:
-                cleaned = uninstaller.paths
-        >>> if cleaned:
-                print("uninstalled packages: %s" % cleaned)
-        """
-
-        auto_confirm = kwargs.pop("auto_confirm", True)
-        verbose = kwargs.pop("verbose", False)
-        with self.activated():
-            monkey_patch = next(
-                iter(
-                    dist
-                    for dist in self.base_working_set
-                    if dist.project_name == "recursive-monkey-patch"
-                ),
-                None,
-            )
-            if monkey_patch:
-                monkey_patch.activate()
-            dist = next(
-                iter(d for d in self.get_working_set() if d.project_name == pkgname), None
-            )
-            path_set = UninstallPathSet.from_dist(dist)
-            if path_set is not None:
-                path_set.remove(auto_confirm=auto_confirm, verbose=verbose)
-            try:
-                yield path_set
-            except Exception:
-                if path_set is not None:
-                    path_set.rollback()
-            else:
-                if path_set is not None:
-                    path_set.commit()
-            if path_set is None:
-                return
