@@ -7,9 +7,13 @@ from tempfile import NamedTemporaryFile
 
 from pipenv import environments, exceptions
 from pipenv.patched.pip._internal.exceptions import PipError
-from pipenv.patched.pip._vendor import rich
 from pipenv.routines.lock import do_lock
-from pipenv.utils.dependencies import convert_deps_to_pip, is_star
+from pipenv.utils import console, err, fileutils
+from pipenv.utils.dependencies import (
+    expansive_install_req_from_line,
+    get_lockfile_section_using_pipfile_category,
+    install_req_from_pipfile,
+)
 from pipenv.utils.indexes import get_source_list
 from pipenv.utils.internet import download_file, is_valid_url
 from pipenv.utils.pip import (
@@ -19,42 +23,32 @@ from pipenv.utils.pip import (
 from pipenv.utils.pipfile import ensure_pipfile
 from pipenv.utils.project import ensure_project
 from pipenv.utils.requirements import add_index_to_pipfile, import_requirements
+from pipenv.utils.shell import temp_environ
 from pipenv.utils.virtualenv import cleanup_virtualenv, do_create_virtualenv
-from pipenv.vendor import click
-from pipenv.vendor.requirementslib import fileutils
-from pipenv.vendor.requirementslib.models.requirements import Requirement
-from pipenv.vendor.requirementslib.utils import temp_environ
-
-console = rich.console.Console()
-err = rich.console.Console(stderr=True)
 
 
 def do_install(
     project,
     packages=False,
     editable_packages=False,
-    index_url=False,
+    index=False,
     dev=False,
     python=False,
     pypi_mirror=None,
     system=False,
     ignore_pipfile=False,
-    skip_lock=False,
     requirementstxt=False,
     pre=False,
     deploy=False,
-    keep_outdated=False,
-    selective_upgrade=False,
     site_packages=None,
     extra_pip_args=None,
     categories=None,
+    skip_lock=False,
 ):
     requirements_directory = fileutils.create_tracked_tempdir(
         suffix="-requirements", prefix="pipenv-"
     )
     warnings.filterwarnings("default", category=ResourceWarning)
-    if selective_upgrade:
-        keep_outdated = True
     packages = packages if packages else []
     editable_packages = editable_packages if editable_packages else []
     package_args = [p for p in packages if p] + [p for p in editable_packages if p]
@@ -72,6 +66,7 @@ def do_install(
         skip_requirements=skip_requirements,
         pypi_mirror=pypi_mirror,
         site_packages=site_packages,
+        categories=categories,
     )
     # Don't attempt to install develop and default packages if Pipfile is missing
     if not project.pipfile_exists and not (package_args or dev):
@@ -82,8 +77,6 @@ def do_install(
     # Load the --pre settings from the Pipfile.
     if not pre:
         pre = project.settings.get("allow_prereleases")
-    if not keep_outdated:
-        keep_outdated = project.settings.get("keep_outdated")
     remote = requirementstxt and is_valid_url(requirementstxt)
     if "default" in categories:
         raise exceptions.PipenvUsageError(
@@ -104,10 +97,9 @@ def do_install(
         os.environ["PIPENV_USE_SYSTEM"] = "1"
     # Check if the file is remote or not
     if remote:
-        click.secho(
+        err.print(
             "Remote requirements file provided! Downloading...",
-            bold=True,
-            err=True,
+            style="bold",
         )
         fd = NamedTemporaryFile(
             prefix="pipenv-", suffix="-requirement.txt", dir=requirements_directory
@@ -120,33 +112,32 @@ def do_install(
         except OSError:
             fd.close()
             os.unlink(temp_reqs)
-            click.secho(
+            err.print(
                 f"Unable to find requirements file at {requirements_url}.",
-                fg="red",
-                err=True,
+                style="red",
             )
             sys.exit(1)
         finally:
             fd.close()
         # Replace the url with the temporary requirements file
         requirementstxt = temp_reqs
-        remote = True
     if requirementstxt:
         error, traceback = None, None
-        click.secho(
+        err.print(
             "Requirements file provided! Importing into Pipfile...",
-            bold=True,
-            err=True,
+            style="bold",
         )
         try:
-            import_requirements(project, r=project.path_to(requirementstxt), dev=dev)
+            import_requirements(
+                project,
+                r=project.path_to(requirementstxt),
+                dev=dev,
+                categories=categories,
+            )
         except (UnicodeDecodeError, PipError) as e:
             # Don't print the temp file path if remote since it will be deleted.
-            req_path = requirements_url if remote else project.path_to(requirementstxt)
-            error = (
-                "Unexpected syntax in {}. Are you sure this is a "
-                "requirements.txt style file?".format(req_path)
-            )
+            req_path = project.path_to(requirementstxt)
+            error = f"Unexpected syntax in {req_path}. Are you sure this is a requirements.txt style file?"
             traceback = e
         except AssertionError as e:
             error = (
@@ -155,41 +146,19 @@ def do_install(
             )
             traceback = e
         finally:
-            # If requirements file was provided by remote url delete the temporary file
-            if remote:
-                fd.close()  # Close for windows to allow file cleanup.
-                os.remove(temp_reqs)
             if error and traceback:
-                click.secho(error, fg="red")
-                click.secho(str(traceback), fg="yellow", err=True)
+                console.print(error, style="red")
+                err.print(str(traceback), style="yellow")
                 sys.exit(1)
 
     # Allow more than one package to be provided.
-    package_args = [p for p in packages] + [f"-e {pkg}" for pkg in editable_packages]
-    # Support for --selective-upgrade.
-    # We should do this part first to make sure that we actually do selectively upgrade
-    # the items specified
-    if selective_upgrade:
-        from pipenv.vendor.requirementslib.models.requirements import Requirement
-
-        for i, package in enumerate(package_args[:]):
-            section = project.packages if not dev else project.dev_packages
-            package = Requirement.from_line(package)
-            package__name, package__val = package.pipfile_entry
-            try:
-                if not is_star(section[package__name]) and is_star(package__val):
-                    # Support for VCS dependencies.
-                    package_args[i] = convert_deps_to_pip(
-                        {package__name: section[package__name]}, project=project
-                    )[0]
-            except KeyError:
-                pass
+    package_args = list(packages) + [f"-e {pkg}" for pkg in editable_packages]
     # Install all dependencies, if none was provided.
     # This basically ensures that we have a pipfile and lockfile, then it locks and
     # installs from the lockfile
     new_packages = []
     if not packages and not editable_packages:
-        # Update project settings with pre preference.
+        # Update project settings with prerelease preference.
         if pre:
             project.update_settings({"allow_prereleases": pre})
         do_init(
@@ -198,20 +167,17 @@ def do_install(
             allow_global=system,
             ignore_pipfile=ignore_pipfile,
             system=system,
-            skip_lock=skip_lock,
             deploy=deploy,
             pre=pre,
             requirements_dir=requirements_directory,
             pypi_mirror=pypi_mirror,
-            keep_outdated=keep_outdated,
             extra_pip_args=extra_pip_args,
             categories=categories,
+            skip_lock=skip_lock,
         )
 
     # This is for if the user passed in dependencies, then we want to make sure we
     else:
-        from pipenv.vendor.requirementslib.models.requirements import Requirement
-
         # make a tuple of (display_name, entry)
         pkg_list = packages + [f"-e {pkg}" for pkg in editable_packages]
         if not system and not project.virtualenv_exists:
@@ -220,20 +186,18 @@ def do_install(
                 dev=dev,
                 system=system,
                 allow_global=system,
-                keep_outdated=keep_outdated,
                 requirements_dir=requirements_directory,
                 deploy=deploy,
                 pypi_mirror=pypi_mirror,
-                skip_lock=skip_lock,
                 extra_pip_args=extra_pip_args,
                 categories=categories,
+                skip_lock=skip_lock,
             )
 
         for pkg_line in pkg_list:
-            click.secho(
+            console.print(
                 f"Installing {pkg_line}...",
-                fg="green",
-                bold=True,
+                style="bold green",
             )
             # pip install:
             with temp_environ(), console.status(
@@ -243,11 +207,13 @@ def do_install(
                     os.environ["PIP_USER"] = "0"
                     if "PYTHONHOME" in os.environ:
                         del os.environ["PYTHONHOME"]
-                st.console.print(f"Resolving {pkg_line}...")
+                st.console.print(f"Resolving {pkg_line}...", markup=False)
                 try:
-                    pkg_requirement = Requirement.from_line(pkg_line)
+                    pkg_requirement, _ = expansive_install_req_from_line(
+                        pkg_line, expand_env=True
+                    )
                 except ValueError as e:
-                    err.print("{}: {}".format(click.style("WARNING", fg="red"), e))
+                    err.print(f"[red]WARNING[/red]: {e}")
                     err.print(
                         environments.PIPENV_SPINNER_FAIL_TEXT.format(
                             "Installation Failed"
@@ -255,21 +221,6 @@ def do_install(
                     )
                     sys.exit(1)
                 st.update(f"Installing {pkg_requirement.name}...")
-                # Warn if --editable wasn't passed.
-                if (
-                    pkg_requirement.is_vcs
-                    and not pkg_requirement.editable
-                    and not project.s.PIPENV_RESOLVE_VCS
-                ):
-                    err.print(
-                        "{}: You installed a VCS dependency in non-editable mode. "
-                        "This will work fine, but sub-dependencies will not be resolved by {}."
-                        "\n  To enable this sub-dependency functionality, specify that this dependency is editable."
-                        "".format(
-                            click.style("Warning", fg="red", bold=True),
-                            click.style("$ pipenv lock", fg="yellow"),
-                        )
-                    )
                 if categories:
                     pipfile_sections = ""
                     for c in categories:
@@ -278,34 +229,42 @@ def do_install(
                     pipfile_sections = "[dev-packages]"
                 else:
                     pipfile_sections = "[packages]"
-                st.console.print(
-                    f"[bold]Adding [green]{pkg_requirement.name}[/green][/bold] to Pipfile's [yellow]\\{pipfile_sections}[/yellow] ..."
-                )
                 # Add the package to the Pipfile.
-                if index_url:
-                    index_name = add_index_to_pipfile(project, index_url)
-                    pkg_requirement.index = index_name
+                if index:
+                    source = project.get_index_by_name(index)
+                    default_index = project.get_default_index()["name"]
+                    if not source:
+                        index_name = add_index_to_pipfile(project, index)
+                        if index_name != default_index:
+                            pkg_requirement.index = index_name
+                    elif source["name"] != default_index:
+                        pkg_requirement.index = source["name"]
                 try:
                     if categories:
                         for category in categories:
-                            added, cat = project.add_package_to_pipfile(
-                                pkg_requirement, dev, category
+                            added, cat, normalized_name = project.add_package_to_pipfile(
+                                pkg_requirement, pkg_line, dev, category
                             )
                             if added:
-                                new_packages.append((pkg_requirement.name, cat))
+                                new_packages.append((normalized_name, cat))
+                                st.console.print(
+                                    f"[bold]Added [green]{normalized_name}[/green][/bold] to Pipfile's "
+                                    f"[yellow]\\{pipfile_sections}[/yellow] ..."
+                                )
                     else:
-                        added, cat = project.add_package_to_pipfile(pkg_requirement, dev)
+                        added, cat, normalized_name = project.add_package_to_pipfile(
+                            pkg_requirement, pkg_line, dev
+                        )
                         if added:
-                            new_packages.append((pkg_requirement.name, cat))
+                            new_packages.append((normalized_name, cat))
+                            st.console.print(
+                                f"[bold]Added [green]{normalized_name}[/green][/bold] to Pipfile's "
+                                f"[yellow]\\{pipfile_sections}[/yellow] ..."
+                            )
                 except ValueError:
                     import traceback
 
-                    err.print(
-                        "{} {}".format(
-                            click.style("Error:", fg="red", bold=True),
-                            traceback.format_exc(),
-                        )
-                    )
+                    err.print(f"[bold][red]Error:[/red][/bold] {traceback.format_exc()}")
                     err.print(
                         environments.PIPENV_SPINNER_FAIL_TEXT.format(
                             "Failed adding package to Pipfile"
@@ -324,19 +283,18 @@ def do_install(
                 dev=dev,
                 system=system,
                 allow_global=system,
-                keep_outdated=keep_outdated,
                 requirements_dir=requirements_directory,
                 deploy=deploy,
                 pypi_mirror=pypi_mirror,
-                skip_lock=skip_lock,
                 extra_pip_args=extra_pip_args,
                 categories=categories,
+                skip_lock=skip_lock,
             )
-        except RuntimeError:
+        except Exception as e:
             # If we fail to install, remove the package from the Pipfile.
             for pkg_name, category in new_packages:
                 project.remove_package_from_pipfile(pkg_name, category)
-            sys.exit(1)
+            raise e
     sys.exit(0)
 
 
@@ -345,7 +303,6 @@ def do_sync(
     dev=False,
     python=None,
     bare=False,
-    dont_upgrade=False,
     user=False,
     clear=False,
     unused=False,
@@ -392,7 +349,7 @@ def do_sync(
         categories=categories,
     )
     if not bare:
-        click.echo(click.style("All dependencies are now up-to-date!", fg="green"))
+        console.print("[green]All dependencies are now up-to-date![/green]")
 
 
 def do_install_dependencies(
@@ -402,11 +359,11 @@ def do_install_dependencies(
     bare=False,
     allow_global=False,
     ignore_hashes=False,
-    skip_lock=False,
     requirements_dir=None,
     pypi_mirror=None,
     extra_pip_args=None,
     categories=None,
+    skip_lock=False,
 ):
     """
     Executes the installation functionality.
@@ -421,37 +378,50 @@ def do_install_dependencies(
         else:
             categories = ["packages"]
 
-    lockfile = None
-    pipfile = None
     for category in categories:
         # Load the lockfile if it exists, or if dev_only is being used.
+        lockfile = None
+        pipfile = None
         if skip_lock:
+            ignore_hashes = True
             if not bare:
-                click.secho("Installing dependencies from Pipfile...", bold=True)
+                console.print("Installing dependencies from Pipfile...", style="bold")
             pipfile = project.get_pipfile_section(category)
         else:
             lockfile = project.get_or_create_lockfile(categories=categories)
             if not bare:
-                click.secho(
-                    "Installing dependencies from Pipfile.lock ({})...".format(
-                        lockfile["_meta"].get("hash", {}).get("sha256")[-6:]
-                    ),
-                    bold=True,
+                console.print(
+                    f"Installing dependencies from Pipfile.lock "
+                    f"({lockfile['_meta'].get('hash', {}).get('sha256')[-6:]})...",
+                    style="bold",
                 )
         dev = dev or dev_only
-        if lockfile:
+        if skip_lock:
+            deps_list = []
+            for req_name, pipfile_entry in pipfile.items():
+                install_req, markers, req_line = install_req_from_pipfile(
+                    req_name, pipfile_entry
+                )
+                req_line = f"{req_line}; {markers}" if markers else f"{req_line}"
+                deps_list.append(
+                    (
+                        install_req,
+                        req_line,
+                    )
+                )
+        else:
             deps_list = list(
                 lockfile.get_requirements(dev=dev, only=dev_only, categories=[category])
             )
-        else:
-            deps_list = []
-            for req_name, specifier in pipfile.items():
-                deps_list.append(Requirement.from_pipfile(req_name, specifier))
-        failed_deps_queue = queue.Queue()
-        if skip_lock:
-            ignore_hashes = True
-        editable_or_vcs_deps = [dep for dep in deps_list if (dep.editable or dep.vcs)]
-        normal_deps = [dep for dep in deps_list if not (dep.editable or dep.vcs)]
+        editable_or_vcs_deps = [
+            (dep, pip_line) for dep, pip_line in deps_list if (dep.link and dep.editable)
+        ]
+        normal_deps = [
+            (dep, pip_line)
+            for dep, pip_line in deps_list
+            if not (dep.link and dep.editable)
+        ]
+
         install_kwargs = {
             "no_deps": not skip_lock,
             "ignore_hashes": ignore_hashes,
@@ -460,51 +430,22 @@ def do_install_dependencies(
             "sequential_deps": editable_or_vcs_deps,
             "extra_pip_args": extra_pip_args,
         }
-
+        if skip_lock:
+            lockfile_section = pipfile
+        else:
+            lockfile_category = get_lockfile_section_using_pipfile_category(category)
+            lockfile_section = lockfile[lockfile_category]
         batch_install(
             project,
             normal_deps,
+            lockfile_section,
             procs,
-            failed_deps_queue,
             requirements_dir,
             **install_kwargs,
         )
 
         if not procs.empty():
-            _cleanup_procs(project, procs, failed_deps_queue)
-
-        # Iterate over the hopefully-poorly-packaged dependencies...
-        if not failed_deps_queue.empty():
-            click.secho("Installing initially failed dependencies...", bold=True)
-            retry_list = []
-            while not failed_deps_queue.empty():
-                failed_dep = failed_deps_queue.get()
-                retry_list.append(failed_dep)
-            install_kwargs.update({"retry": False})
-            batch_install(
-                project,
-                retry_list,
-                procs,
-                failed_deps_queue,
-                requirements_dir,
-                **install_kwargs,
-            )
-        if not procs.empty():
-            _cleanup_procs(project, procs, failed_deps_queue, retry=False)
-        if not failed_deps_queue.empty():
-            failed_list = []
-            while not failed_deps_queue.empty():
-                failed_dep = failed_deps_queue.get()
-                failed_list.append(failed_dep)
-            click.echo(
-                click.style(
-                    f"Failed to install some dependency or packages.  "
-                    f"The following have failed installation and attempted retry: {failed_list}",
-                    fg="red",
-                ),
-                err=True,
-            )
-            sys.exit(1)
+            _cleanup_procs(project, procs)
 
 
 def batch_install_iteration(
@@ -512,33 +453,12 @@ def batch_install_iteration(
     deps_to_install,
     sources,
     procs,
-    failed_deps_queue,
     requirements_dir,
     no_deps=True,
     ignore_hashes=False,
     allow_global=False,
-    retry=True,
     extra_pip_args=None,
 ):
-    from pipenv.vendor.requirementslib.models.utils import (
-        strip_extras_markers_from_requirement,
-    )
-
-    is_artifact = False
-    for dep in deps_to_install:
-        if dep.req.req:
-            dep.req.req = strip_extras_markers_from_requirement(dep.req.req)
-        if dep.markers:
-            dep.markers = str(strip_extras_markers_from_requirement(dep.get_markers()))
-        # Install the module.
-        if dep.is_file_or_url and (
-            dep.is_direct_url
-            or any(dep.req.uri.endswith(ext) for ext in ["zip", "tar.gz"])
-        ):
-            is_artifact = True
-        elif dep.is_vcs:
-            is_artifact = True
-
     with temp_environ():
         if not allow_global:
             os.environ["PIP_USER"] = "0"
@@ -546,10 +466,6 @@ def batch_install_iteration(
                 del os.environ["PYTHONHOME"]
         if "GIT_CONFIG" in os.environ:
             del os.environ["GIT_CONFIG"]
-        use_pep517 = True
-        if not retry and not is_artifact:
-            use_pep517 = False
-
         cmds = pip_install_deps(
             project,
             deps=deps_to_install,
@@ -558,26 +474,25 @@ def batch_install_iteration(
             ignore_hashes=ignore_hashes,
             no_deps=no_deps,
             requirements_dir=requirements_dir,
-            use_pep517=use_pep517,
+            use_pep517=True,
             extra_pip_args=extra_pip_args,
         )
 
         for c in cmds:
             procs.put(c)
-            _cleanup_procs(project, procs, failed_deps_queue, retry=retry)
+            _cleanup_procs(project, procs)
 
 
 def batch_install(
     project,
     deps_list,
+    lockfile_section,
     procs,
-    failed_deps_queue,
     requirements_dir,
     no_deps=True,
     ignore_hashes=False,
     allow_global=False,
     pypi_mirror=None,
-    retry=True,
     sequential_deps=None,
     extra_pip_args=None,
 ):
@@ -586,7 +501,9 @@ def batch_install(
     deps_to_install = deps_list[:]
     deps_to_install.extend(sequential_deps)
     deps_to_install = [
-        dep for dep in deps_to_install if not project.environment.is_satisfied(dep)
+        (dep, pip_line)
+        for dep, pip_line in deps_to_install
+        if not project.environment.is_satisfied(dep)
     ]
     search_all_sources = project.settings.get("install_search_all_sources", False)
     sources = get_source_list(
@@ -597,27 +514,28 @@ def batch_install(
         pypi_mirror=pypi_mirror,
     )
     if search_all_sources:
+        dependencies = [pip_line for _, pip_line in deps_to_install]
         batch_install_iteration(
             project,
-            deps_to_install,
+            dependencies,
             sources,
             procs,
-            failed_deps_queue,
             requirements_dir,
             no_deps=no_deps,
             ignore_hashes=ignore_hashes,
             allow_global=allow_global,
-            retry=retry,
             extra_pip_args=extra_pip_args,
         )
     else:
         # Sort the dependencies out by index -- include editable/vcs in the default group
         deps_by_index = defaultdict(list)
-        for dependency in deps_to_install:
-            if dependency.index:
-                deps_by_index[dependency.index].append(dependency)
-            else:
-                deps_by_index[project.sources_default["name"]].append(dependency)
+        for dependency, pip_line in deps_to_install:
+            index = project.sources_default["name"]
+            if dependency.name and dependency.name in lockfile_section:
+                entry = lockfile_section[dependency.name]
+                if isinstance(entry, dict) and "index" in entry:
+                    index = entry["index"]
+            deps_by_index[index].append(pip_line)
         # Treat each index as its own pip install phase
         for index_name, dependencies in deps_by_index.items():
             try:
@@ -627,24 +545,21 @@ def batch_install(
                     dependencies,
                     [install_source],
                     procs,
-                    failed_deps_queue,
                     requirements_dir,
                     no_deps=no_deps,
                     ignore_hashes=ignore_hashes,
                     allow_global=allow_global,
-                    retry=retry,
                     extra_pip_args=extra_pip_args,
                 )
-            except StopIteration:
-                click.secho(
+            except StopIteration:  # noqa: PERF203
+                console.print(
                     f"Unable to find {index_name} in sources, please check dependencies: {dependencies}",
-                    fg="red",
-                    bold=True,
+                    style="bold red",
                 )
                 sys.exit(1)
 
 
-def _cleanup_procs(project, procs, failed_deps_queue, retry=True):
+def _cleanup_procs(project, procs):
     while not procs.empty():
         c = procs.get()
         try:
@@ -653,52 +568,17 @@ def _cleanup_procs(project, procs, failed_deps_queue, retry=True):
             out, err = c.stdout, c.stderr
         failed = c.returncode != 0
         if project.s.is_verbose():
-            click.secho(out.strip() or err.strip(), fg="yellow")
+            console.print(out.strip() or err.strip(), style="yellow")
         # The Installation failed...
         if failed:
+            # The Installation failed...
+            # We echo both c.stdout and c.stderr because pip returns error details on out.
+            err = err.strip().splitlines() if err else []
+            out = out.strip().splitlines() if out else []
+            err_lines = [line for message in [out, err] for line in message]
             deps = getattr(c, "deps", {}).copy()
-            for dep in deps:
-                # If there is a mismatch in installed locations or the install fails
-                # due to wrongful disabling of pep517, we should allow for
-                # additional passes at installation
-                if "does not match installed location" in err:
-                    project.environment.expand_egg_links()
-                    click.echo(
-                        "{}".format(
-                            click.style(
-                                "Failed initial installation: Failed to overwrite existing "
-                                "package, likely due to path aliasing. Expanding and trying "
-                                "again!",
-                                fg="yellow",
-                            )
-                        )
-                    )
-                    if dep:
-                        dep.use_pep517 = True
-                elif "Disabling PEP 517 processing is invalid" in err:
-                    if dep:
-                        dep.use_pep517 = True
-                elif not retry:
-                    # The Installation failed...
-                    # We echo both c.stdout and c.stderr because pip returns error details on out.
-                    err = err.strip().splitlines() if err else []
-                    out = out.strip().splitlines() if out else []
-                    err_lines = [line for message in [out, err] for line in message]
-                    # Return the subprocess' return code.
-                    raise exceptions.InstallError(deps, extra=err_lines)
-                else:
-                    # Alert the user.
-                    if dep:
-                        dep.use_pep517 = False
-                    click.echo(
-                        "{} {}! Will try again.".format(
-                            click.style("An error occurred while installing", fg="red"),
-                            click.style(dep.as_line() if dep else "", fg="green"),
-                        ),
-                        err=True,
-                    )
-                # Save the Failed Dependency for later.
-                failed_deps_queue.put(dep)
+            # Return the subprocess' return code.
+            raise exceptions.InstallError(deps, extra=err_lines)
 
 
 def do_init(
@@ -707,15 +587,14 @@ def do_init(
     dev_only=False,
     allow_global=False,
     ignore_pipfile=False,
-    skip_lock=False,
     system=False,
     deploy=False,
     pre=False,
-    keep_outdated=False,
     requirements_dir=None,
     pypi_mirror=None,
     extra_pip_args=None,
     categories=None,
+    skip_lock=False,
 ):
     """Executes the init functionality."""
     python = None
@@ -726,13 +605,12 @@ def do_init(
     if categories is None:
         categories = []
 
-    if not system and not project.s.PIPENV_USE_SYSTEM:
-        if not project.virtualenv_exists:
-            try:
-                do_create_virtualenv(project, python=python, pypi_mirror=pypi_mirror)
-            except KeyboardInterrupt:
-                cleanup_virtualenv(project, bare=False)
-                sys.exit(1)
+    if not system and not project.s.PIPENV_USE_SYSTEM and not project.virtualenv_exists:
+        try:
+            do_create_virtualenv(project, python=python, pypi_mirror=pypi_mirror)
+        except KeyboardInterrupt:
+            cleanup_virtualenv(project, bare=False)
+            sys.exit(1)
     # Ensure the Pipfile exists.
     if not deploy:
         ensure_pipfile(project, system=system)
@@ -746,38 +624,31 @@ def do_init(
         new_hash = project.calculate_pipfile_hash()
         if new_hash != old_hash:
             if deploy:
-                click.secho(
-                    "Your Pipfile.lock ({}) is out of date. Expected: ({}).".format(
-                        old_hash[-6:], new_hash[-6:]
-                    ),
-                    fg="red",
+                console.print(
+                    f"Your Pipfile.lock ({old_hash[-6:]}) is out of date.  Expected: ({new_hash[-6:]}).",
+                    style="red",
                 )
                 raise exceptions.DeployException
-            elif (system or allow_global) and not (project.s.PIPENV_VIRTUALENV):
-                click.secho(
-                    "Pipfile.lock ({}) out of date, but installation "
-                    "uses {} re-building lockfile must happen in "
-                    "isolation. Please rebuild lockfile in a virtualenv. "
-                    "Continuing anyway...".format(old_hash[-6:], "--system"),
-                    fg="yellow",
-                    err=True,
+            if (system or allow_global) and not (project.s.PIPENV_VIRTUALENV):
+                err.print(
+                    f"Pipfile.lock ({old_hash[-6:]}) out of date, but installation uses --system so"
+                    f"re-building lockfile must happen in isolation."
+                    f" Please rebuild lockfile in a virtualenv.  Continuing anyway...",
+                    style="yellow",
                 )
             else:
                 if old_hash:
                     msg = "Pipfile.lock ({0}) out of date, updating to ({1})..."
                 else:
                     msg = "Pipfile.lock is corrupt, replaced with ({1})..."
-                click.secho(
+                err.print(
                     msg.format(old_hash[-6:], new_hash[-6:]),
-                    fg="yellow",
-                    bold=True,
-                    err=True,
+                    style="bold yellow",
                 )
                 do_lock(
                     project,
                     system=system,
                     pre=pre,
-                    keep_outdated=keep_outdated,
                     write=True,
                     pypi_mirror=pypi_mirror,
                     categories=categories,
@@ -794,16 +665,14 @@ def do_init(
                 "See also: --deploy flag.",
             )
         else:
-            click.secho(
+            err.print(
                 "Pipfile.lock not found, creating...",
-                bold=True,
-                err=True,
+                style="bold",
             )
             do_lock(
                 project,
                 system=system,
                 pre=pre,
-                keep_outdated=keep_outdated,
                 write=True,
                 pypi_mirror=pypi_mirror,
                 categories=categories,
@@ -813,20 +682,16 @@ def do_init(
         dev=dev,
         dev_only=dev_only,
         allow_global=allow_global,
-        skip_lock=skip_lock,
         requirements_dir=requirements_dir,
         pypi_mirror=pypi_mirror,
         extra_pip_args=extra_pip_args,
         categories=categories,
+        skip_lock=skip_lock,
     )
 
     # Hint the user what to do to activate the virtualenv.
     if not allow_global and not deploy and "PIPENV_ACTIVE" not in os.environ:
-        click.echo(
-            "To activate this project's virtualenv, run {}.\n"
-            "Alternatively, run a command "
-            "inside the virtualenv with {}.".format(
-                click.style("pipenv shell", fg="yellow"),
-                click.style("pipenv run", fg="yellow"),
-            )
+        console.print(
+            "To activate this project's virtualenv, run [yellow]pipenv shell[/yellow].\n"
+            "Alternatively, run a command inside the virtualenv with [yellow]pipenv run[/yellow]."
         )
