@@ -467,14 +467,14 @@ def convert_deps_to_pip(
     include_index=False,
 ):
     """ "Converts a Pipfile-formatted dependency to a pip-formatted one."""
-    dependencies = []
+    dependencies = {}
     if indexes is None:
         indexes = []
     for dep_name, dep in deps.items():
         req = dependency_as_pip_install_line(
             dep_name, dep, include_hashes, include_markers, include_index, indexes
         )
-        dependencies.append(req)
+        dependencies[dep_name] = req
     return dependencies
 
 
@@ -514,9 +514,23 @@ def parse_pkginfo_file(content: str):
 
 
 def parse_setup_file(content):
+    # A dictionary to store variable names and their values
+    variables = {}
     try:
         tree = ast.parse(content)
         for node in ast.walk(tree):
+            # Extract variable assignments and store them
+            if isinstance(node, ast.Assign):
+                for target in node.targets:
+                    if isinstance(target, ast.Name):
+                        if isinstance(node.value, ast.Str):  # for Python versions < 3.8
+                            variables[target.id] = node.value.s
+                        elif isinstance(node.value, ast.Constant) and isinstance(
+                            node.value.value, str
+                        ):
+                            variables[target.id] = node.value.value
+
+            # Check function calls to extract the 'name' attribute from the setup function
             if isinstance(node, ast.Call):
                 if (
                     getattr(node.func, "id", "") == "setup"
@@ -525,12 +539,20 @@ def parse_setup_file(content):
                 ):
                     for keyword in node.keywords:
                         if keyword.arg == "name":
-                            if isinstance(keyword.value, ast.Str):
+                            # If it's a variable, retrieve its value
+                            if (
+                                isinstance(keyword.value, ast.Name)
+                                and keyword.value.id in variables
+                            ):
+                                return variables[keyword.value.id]
+                            # Otherwise, check if it's directly provided
+                            elif isinstance(keyword.value, ast.Str):
                                 return keyword.value.s
                             elif isinstance(keyword.value, ast.Constant) and isinstance(
                                 keyword.value.value, str
                             ):
                                 return keyword.value.value
+                            # Additional handling for Python versions and specific ways of defining the name
                             elif sys.version_info < (3, 9) and isinstance(
                                 keyword.value, ast.Subscript
                             ):
@@ -546,14 +568,12 @@ def parse_setup_file(content):
                             elif sys.version_info >= (3, 9) and isinstance(
                                 keyword.value, ast.Subscript
                             ):
-                                # If the name is a lookup in a dictionary, only handle the case where it's a static lookup
                                 if (
                                     isinstance(keyword.value.value, ast.Name)
                                     and isinstance(keyword.value.slice, ast.Str)
                                     and keyword.value.value.id == "about"
                                 ):
                                     return keyword.value.slice.s
-
     except ValueError:
         pass  # We will not exec unsafe code to determine the name pre-resolver
 
@@ -640,15 +660,43 @@ def find_package_name_from_directory(directory):
     return None
 
 
+def ensure_path_is_relative(file_path):
+    abs_path = Path(file_path).resolve()
+    current_dir = Path.cwd()
+
+    # Check if the paths are on different drives
+    if abs_path.drive != current_dir.drive:
+        # If on different drives, return the absolute path
+        return str(abs_path)
+
+    try:
+        # Try to create a relative path
+        return str(abs_path.relative_to(current_dir))
+    except ValueError:
+        # If the direct relative_to fails, manually compute the relative path
+        common_parts = 0
+        for part_a, part_b in zip(abs_path.parts, current_dir.parts):
+            if part_a == part_b:
+                common_parts += 1
+            else:
+                break
+
+        # Number of ".." needed are the extra parts in the current directory
+        # beyond the common parts
+        up_levels = [".."] * (len(current_dir.parts) - common_parts)
+        # The relative path is constructed by going up as needed and then
+        # appending the non-common parts of the absolute path
+        rel_parts = up_levels + list(abs_path.parts[common_parts:])
+        relative_path = Path(*rel_parts)
+        return str(relative_path)
+
+
 def determine_path_specifier(package: InstallRequirement):
     if package.link:
         if package.link.scheme in ["http", "https"]:
             return package.link.url_without_fragment
         if package.link.scheme == "file":
-            try:
-                return Path(package.link.file_path).relative_to(Path.cwd()).as_posix()
-            except ValueError:
-                return Path(package.link.file_path).as_posix()
+            return ensure_path_is_relative(package.link.file_path)
 
 
 def determine_vcs_specifier(package: InstallRequirement):
@@ -872,7 +920,7 @@ def expand_env_variables(line) -> AnyStr:
 
 
 def expansive_install_req_from_line(
-    name: str,
+    pip_line: str,
     comes_from: Optional[Union[str, InstallRequirement]] = None,
     *,
     use_pep517: Optional[bool] = None,
@@ -884,30 +932,43 @@ def expansive_install_req_from_line(
     user_supplied: bool = False,
     config_settings: Optional[Dict[str, Union[str, List[str]]]] = None,
     expand_env: bool = False,
-) -> InstallRequirement:
-    """Creates an InstallRequirement from a name, which might be a
-    requirement, directory containing 'setup.py', filename, or URL.
-
-    :param line_source: An optional string describing where the line is from,
-        for logging purposes in case of an error.
+) -> (InstallRequirement, str):
+    """Create an InstallRequirement from a pip-style requirement line.
+    InstallRequirement is a pip internal construct that represents an installable requirement,
+    and is used as an intermediary between the pip command and the resolver.
+    :param pip_line: A pip-style requirement line.
+    :param comes_from: The path to the requirements file the line was found in.
+    :param use_pep517: Whether to use PEP 517/518 when installing the
+        requirement.
+    :param isolated: Whether to isolate the requirements when installing them. (likely unused)
+    :param global_options: Extra global options to be used when installing the install req (likely unused)
+    :param hash_options: Extra hash options to be used when installing the install req (likely unused)
+    :param constraint: Whether the requirement is a constraint.
+    :param line_source: The source of the line (e.g. "requirements.txt").
+    :param user_supplied: Whether the requirement was directly provided by the user.
+    :param config_settings: Configuration settings to be used when installing the install req (likely unused)
+    :param expand_env: Whether to expand environment variables in the line. (definitely used)
+    :return: A tuple of the InstallRequirement and the name of the package (if determined).
     """
-    name = name.strip("'")
-    if name.startswith("-e "):  # Editable requirements
-        name = name.split("-e ")[1]
-        return install_req_from_editable(name, line_source)
-    if has_name_with_extras(name):
-        name = name.split(" @ ", 1)[1]
+    name = None
+    pip_line = pip_line.strip("'").lstrip(" ")
+    for new_req_symbol in ("@ ", " @ "):  # Check for new style pip lines
+        if new_req_symbol in pip_line:
+            pip_line_parts = pip_line.split(new_req_symbol, 1)
+            name = pip_line_parts[0]
+            pip_line = pip_line_parts[1]
+    if pip_line.startswith("-e "):  # Editable requirements
+        pip_line = pip_line.split("-e ")[1]
+        return install_req_from_editable(pip_line, line_source), name
 
     if expand_env:
-        name = expand_env_variables(name)
+        pip_line = expand_env_variables(pip_line)
 
-    vcs_part = name
-    if "@ " in name:  # Check for new style vcs lines
-        vcs_part = name.split("@ ", 1)[1]
+    vcs_part = pip_line
     for vcs in VCS_LIST:
         if vcs_part.startswith(f"{vcs}+"):
             link = get_link_from_line(vcs_part)
-            return InstallRequirement(
+            install_req = InstallRequirement(
                 None,
                 comes_from,
                 link=link,
@@ -918,22 +979,23 @@ def expansive_install_req_from_line(
                 constraint=constraint,
                 user_supplied=user_supplied,
             )
-    if urlparse(name).scheme in ("http", "https", "file") or any(
-        name.endswith(s) for s in INSTALLABLE_EXTENSIONS
+            return install_req, name
+    if urlparse(pip_line).scheme in ("http", "https", "file") or any(
+        pip_line.endswith(s) for s in INSTALLABLE_EXTENSIONS
     ):
-        parts = parse_req_from_line(name, line_source)
+        parts = parse_req_from_line(pip_line, line_source)
     else:
         # It's a requirement
-        if "--index" in name:
-            name = name.split("--index")[0]
-        if " -i " in name:
-            name = name.split(" -i ")[0]
+        if "--index" in pip_line:
+            pip_line = pip_line.split("--index")[0]
+        if " -i " in pip_line:
+            pip_line = pip_line.split(" -i ")[0]
         # handle local version identifiers (like the ones torch uses in their public index)
-        if "+" in name:
-            name = name.split("+")[0]
-        parts = parse_req_from_line(name, line_source)
+        if "+" in pip_line:
+            pip_line = pip_line.split("+")[0]
+        parts = parse_req_from_line(pip_line, line_source)
 
-    return InstallRequirement(
+    install_req = InstallRequirement(
         parts.requirement,
         comes_from,
         link=parts.link,
@@ -947,9 +1009,47 @@ def expansive_install_req_from_line(
         extras=parts.extras,
         user_supplied=user_supplied,
     )
+    return install_req, name
+
+
+def file_path_from_pipfile(path_str, pipfile_entry):
+    """Creates an installable file path from a pipfile entry.
+    Handles local and remote paths, files and directories;
+    supports extras and editable specification.
+    Outputs a pip installable line.
+    """
+    if path_str.startswith(("http:", "https:", "ftp:")):
+        req_str = path_str
+    else:
+        req_str = ensure_path_is_relative(path_str)
+
+    if pipfile_entry.get("extras"):
+        req_str = f"{req_str}[{','.join(pipfile_entry['extras'])}]"
+    if pipfile_entry.get("editable", False):
+        req_str = f"-e {req_str}"
+
+    return req_str
+
+
+def normalize_vcs_url(vcs_url):
+    """Return vcs_url and possible vcs_ref from a given vcs_url."""
+    # We have to handle the fact that some vcs urls have a ref in them
+    # and some have a netloc with a username and password in them, and some have both
+    vcs_ref = ""
+    if "@" in vcs_url:
+        parsed_url = urlparse(vcs_url)
+        if "@" in parsed_url.path:
+            url_parts = vcs_url.rsplit("@", 1)
+            vcs_url = url_parts[0]
+            vcs_ref = url_parts[1]
+    return vcs_url, vcs_ref
 
 
 def install_req_from_pipfile(name, pipfile):
+    """Creates an InstallRequirement from a name and a pipfile entry.
+    Handles VCS, local & remote paths, and regular named requirements.
+    "file" and "path" entries are treated the same.
+    """
     _pipfile = {}
     vcs = None
     if hasattr(pipfile, "keys"):
@@ -971,27 +1071,20 @@ def install_req_from_pipfile(name, pipfile):
         subdirectory = _pipfile.get("subdirectory", "")
         if subdirectory:
             subdirectory = f"#subdirectory={subdirectory}"
-        fallback_ref = ""
-        if ("ssh://" in vcs_url and vcs_url.count("@") >= 2) or (
-            "ssh://" not in vcs_url and "@" in vcs_url
-        ):
-            vcs_url_parts = vcs_url.rsplit("@", 1)
-            if re.match(r"^[\w\.]+$", vcs_url_parts[1]):
-                vcs_url = vcs_url_parts[0]
-                fallback_ref = vcs_url_parts[1]
+        vcs_url, fallback_ref = normalize_vcs_url(vcs_url)
         req_str = f"{vcs_url}@{_pipfile.get('ref', fallback_ref)}{extras_str}"
         if not req_str.startswith(f"{vcs}+"):
             req_str = f"{vcs}+{req_str}"
-        if f"{vcs}+file://" in req_str:
+        if f"{vcs}+file://" in req_str or _pipfile.get("editable", False):
             req_str = (
                 f"-e {req_str}#egg={name}{extras_str}{subdirectory.replace('#', '&')}"
             )
         else:
             req_str = f"{name}{extras_str}@ {req_str}{subdirectory}"
     elif "path" in _pipfile:
-        req_str = str(Path(_pipfile["path"]).as_posix())
+        req_str = file_path_from_pipfile(_pipfile["path"], _pipfile)
     elif "file" in _pipfile:
-        req_str = str(Path(_pipfile["file"]).as_posix())
+        req_str = file_path_from_pipfile(_pipfile["file"], _pipfile)
     else:
         # We ensure version contains an operator. Default to equals (==)
         _pipfile["version"] = version = get_version(pipfile)
@@ -1001,7 +1094,7 @@ def install_req_from_pipfile(name, pipfile):
             version = ""
         req_str = f"{name}{extras_str}{version}"
 
-    install_req = expansive_install_req_from_line(
+    install_req, _ = expansive_install_req_from_line(
         req_str,
         comes_from=None,
         use_pep517=False,
@@ -1114,9 +1207,7 @@ def is_required_version(version, specified_version):
 
 def is_editable(pipfile_entry):
     if hasattr(pipfile_entry, "get"):
-        return pipfile_entry.get("editable", False) or any(
-            pipfile_entry.get(key) for key in ("file", "path") + VCS_LIST
-        )
+        return pipfile_entry.get("editable", False)
     return False
 
 
