@@ -3,7 +3,8 @@ import os
 import pytest
 
 from pipenv.utils.shell import temp_environ
-from pipenv.routines.requirements import requirements_from_deps
+from pipenv.utils.requirements import requirements_from_lockfile
+
 
 @pytest.mark.requirements
 def test_requirements_generates_requirements_from_lockfile(pipenv_instance_pypi):
@@ -289,11 +290,11 @@ def test_requirements_generates_requirements_from_lockfile_without_env_var_expan
             },
             True,
             True,
-            ["pyjwt[crypto] @ git+https://github.com/jpadilla/pyjwt.git@7665aa625506a11bae50b56d3e04413a3dc6fdf8"]
+            ["pyjwt[crypto]@ git+https://github.com/jpadilla/pyjwt.git@7665aa625506a11bae50b56d3e04413a3dc6fdf8"]
         )
     ]
 )
 def test_requirements_from_deps(deps, include_hashes, include_markers, expected):
-    result = requirements_from_deps(deps, include_hashes, include_markers)
+    result = requirements_from_lockfile(deps, include_hashes, include_markers)
     assert result == expected
 
